@@ -18,13 +18,13 @@ Notes
 import argparse
 import binascii
 import os
-import psutil
 
 from ipywidgets import DOMWidget, register
 from traitlets import Int, Unicode
 from ._frontend import module_name, module_version
 
 from .server import start_webserver
+from .utility import find_next_free_port
 from wslink import server
 server.start_webserver = start_webserver
 
@@ -42,27 +42,40 @@ class RemoteRenderer(DOMWidget):
 
     Parameters
     ----------
-    pvserver_host: str
-        if not None, create a connection to a pvserver at the given host.
+    pvserverHost: str
+        if not None, create a connection to a pvserver at the given host
+        upon start of the ParaViewWeb server.
         Default = None
-    pvserver_port: str
+    pvserverPort: str
         port on which the pvserver is listening 
         Default = 11111
+    viewID: str
+        viewID of the view to connect to 
+        (only relevant if multiple views exist on the server side)
+        Default = "-1"
     baseURL: str
         the baseURL under which the ParaViewWeb server will be started.
         If the notebook is running under `http://localhost:8888`, 
         the baseURL would be `localhost`.
         If Jupyter Server Proxy is being used, the baseURL is the part
-        in front of 'proxy'. So if your notebook url is `http://localhost:8888`
-        and you would access a process using `http://localhost:88888/proxy/8080`,
+        in front of "proxy". So if your notebook url is `http://localhost:8888`
+        and you would access a process using `http://localhost:8888/proxy/8080`,
         the baseURL would be `localhost:8888`.
         Default = "localhost"
-    use_jupyter_server_proxy: bool
+    useJupyterServerProxy: bool
         whether the connection should be established using Jupyter Server Proxy.
         If True, the baseURL needs to be adjusted accordingly.
         Default = False
-    use_jupyter_server_proxy_https: bool
-        Access using wss, to Jupyter server proxy. There it will be decrypted and send as ws to socket
+    useJupyterServerProxyHttps: bool
+        whether the connection should be established using Jupyter Server Proxy
+        using wss. Use this if your notebook server is running on HTTPS.
+        Beware, communication behind the proxy will still be unencrypted if
+        your webserver was not started on HTTPS (using a SSL key and certificate).
+        Default = False
+    disableExternalPort: bool
+        Disables port checking during the WebSocket opening handshake.
+        Needed when using Jupyter Server Proxy to prevent a port mismatch error 
+        since the notebook and the webserver do not run on the same port.
         Default = False
     protocol: pv_wslink.PVServerProtocol
         a custom PVServerProtocol class which handles clients requests and run
@@ -103,18 +116,20 @@ class RemoteRenderer(DOMWidget):
     # Placeholder to force rendering updates on change.
     _update = Int(0).tag(sync=True)
 
-    def __init__(self, pvserver_host=None, pvserver_port=11111, baseURL='localhost', use_jupyter_server_proxy=False, use_jupyter_server_proxy_https=False, protocol=None, *args, **kwargs):
+    def __init__(self, pvserverHost=None, pvserverPort=11111, viewID="-1", baseURL='localhost', useJupyterServerProxy=False, useJupyterServerProxyHttps=False, disableExternalPort=False, protocol=None, *args, **kwargs):
         super().__init__(*args, **kwargs)
         self.baseURL = baseURL
-        self.use_jupyter_server_proxy = use_jupyter_server_proxy
-        self.use_jupyter_server_proxy_https = use_jupyter_server_proxy_https
-        self.pvserver_host = pvserver_host
-        self.pvserver_port = pvserver_port
+        self.viewID = viewID
+        self.useJupyterServerProxy = useJupyterServerProxy
+        self.useJupyterServerProxyHttps = useJupyterServerProxyHttps
+        self.disableExternalPort = disableExternalPort
+        self.pvserverHost = pvserverHost
+        self.pvserverPort = pvserverPort
         self.protocol = protocol
 
-        self.start_webserver(self.protocol, **kwargs)
+        self.start_webserver(**kwargs)
 
-    def start_webserver(self, protocol=None, **kwargs):
+    def start_webserver(self, **kwargs):
         """Start a ParaViewWeb server with the given arguments."""
         from paraview.web import pv_wslink
         from paraview.web import protocols as pv_protocols
@@ -137,8 +152,7 @@ class RemoteRenderer(DOMWidget):
                     from paraview.simple import Connect as PVConnect
                     PVConnect(_CustomServer.pv_host, _CustomServer.pv_port)
         
-        if protocol == None:
-            protocol = _CustomServer
+        protocol = _CustomServer if self.protocol == None else self.protocol
 
         # Create argument parser and add arguments (imitates command line)
         parser = argparse.ArgumentParser(description="ParaView Web Server")
@@ -163,7 +177,7 @@ class RemoteRenderer(DOMWidget):
                 arg_list.append(value)
         # If no port is given, check for the next free port starting from 8080
         if 'p' not in kwargs.keys() and 'port' not in kwargs.keys():
-            port = _find_next_free_port(8080)
+            port = find_next_free_port(8080)
             arg_list.append('-p')
             arg_list.append(str(port))
         # If no authKey is given, create a randon authentication key
@@ -172,58 +186,35 @@ class RemoteRenderer(DOMWidget):
             arg_list.append('-a')
             arg_list.append(authKey)
         args = parser.parse_args(arg_list)
-        # print(args)
         
         # Start server
         if protocol == _CustomServer:
             _CustomServer.authKey = args.authKey
-            if self.pvserver_host is not None:
-                _CustomServer.pv_host = self.pvserver_host
-                _CustomServer.pv_port = self.pvserver_port
+            if self.pvserverHost is not None:
+                _CustomServer.pv_host = self.pvserverHost
+                _CustomServer.pv_port = self.pvserverPort
         else:
             protocol.authKey = args.authKey
-        self.protocol = server.start_webserver(options=args, protocol=protocol)
-        # Set authKey and URL for the websocket connection
+        self.protocol = server.start_webserver(options=args, disableExternalPort=self.disableExternalPort, protocol=protocol)
+        # Set authKey and URL for the WebSocket connection
         self.authKey = args.authKey
         self.port = args.port
         self.sessionURL = '{wsProtocol}://{baseURL}{use_proxy}{port}/{ws_endpoint}'.format(
-            wsProtocol='wss' if (args.sslKey and args.sslCert) or self.use_jupyter_server_proxy_https else 'ws',
-            use_proxy='/proxy/' if self.use_jupyter_server_proxy else ':',
+            wsProtocol='wss' if (args.sslKey and args.sslCert) or self.useJupyterServerProxyHttps else 'ws',
+            use_proxy='/proxy/' if self.useJupyterServerProxy or self.useJupyterServerProxyHttps else ':',
             baseURL=self.baseURL, port=args.port, ws_endpoint=args.ws
         )
 
-    def _find_next_free_port(start_port):
-        """
-        Finds next free port starting from a given port using the psutil module.
-        For port numbers smaller than 1, starts search with the first registered port (1024),
-        returns None, if no free port can be found between start_port and the biggest port number (65535)
-        """
-        max_port = 65535
-
-        if start_port < 1:
-            port = 1024
-        else:
-            port = start_port
-
-        while port <= max_port:
-            for conn in psutil.net_connections():
-                if conn.status == 'LISTEN' and conn.laddr.port == port:
-                    port += 1
-                    break
-                else:
-                    return port
-        return None
-
     def update_render(self):
         """Explicit call for the renderer on the javascript side to render."""
         self._update += 1
         
     def get_webserver_port(self):
-      """Get the port, that can be used to reach the websocket."""
+      """Get the port, that can be used to reach the WebSocket."""
       return self.port
         
     def get_authKey(self):
-      """Get the key, used for authentification for the websocket connection."""
+      """Get the key, used for authentification for the WebSocket connection."""
       return self.authKey
 
     def webserver_arguments_help():
