@@ -32,7 +32,8 @@ class SimpleRenderer(DOMWidget):
             authentication key for clients to connect to the server
             Default = "wslink-secret"
         viewID: str
-            viewID of the view to connect to (only relevant if multiple views exist on the server side)
+            viewID of the view to connect to 
+            (only relevant if multiple views exist on the server side)
             Default = "-1"
 
     Examples
